package org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener;

import com.google.common.base.Objects;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Consumer;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.gemoc.commons.eclipse.emf.EMFResource;
import org.gemoc.xdsmlframework.api.core.IBasicExecutionEngine;
import org.gemoc.xdsmlframework.api.core.IExecutionContext;
import org.gemoc.xdsmlframework.api.core.IExecutionPlatform;
import org.gemoc.xdsmlframework.api.engine_addon.DefaultEngineAddon;
import org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener.ModelChange;
import org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener.NewObjectModelChange;
import org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener.NonCollectionFieldModelChange;
import org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener.PotentialCollectionFieldModelChange;
import org.gemoc.xdsmlframework.api.engine_addon.modelchangelistener.RemovedObjectModelChange;

/**
 * This model listener gathers EMF notifications, and computes when asked a
 * set of ModelChange objects to reflect what happened in a more abstract
 * and concise way.
 * 
 * For instance, if a field changed multiple times in between two queries to
 * the model listener, a single ModelChange object will be computed to reflect that change,
 * instead of a list of many EMF Notifications.
 * 
 * A ModelChange can be a new/removed object in the model, or a change in a field.
 * See associated class.
 * 
 * TODO: detect new/removed objects from the model resources roots.
 */
@SuppressWarnings("all")
public class BatchModelChangeListenerAddon extends DefaultEngineAddon {
  private EContentAdapter adapter;
  
  private IBasicExecutionEngine engine;
  
  private Map<Object, List<Notification>> changes = new HashMap<Object, List<Notification>>();
  
  private Set<Object> registeredObservers = new HashSet<Object>();
  
  public BatchModelChangeListenerAddon(final IBasicExecutionEngine engine) {
    this.engine = engine;
    IExecutionContext _executionContext = engine.getExecutionContext();
    IExecutionPlatform _executionPlatform = _executionContext.getExecutionPlatform();
    _executionPlatform.addEngineAddon(this);
    this.adapter = new EContentAdapter() {
      @Override
      public void notifyChanged(final Notification notification) {
        super.notifyChanged(notification);
        for (final Object obs : BatchModelChangeListenerAddon.this.registeredObservers) {
          List<Notification> _get = BatchModelChangeListenerAddon.this.changes.get(obs);
          _get.add(notification);
        }
      }
    };
    IExecutionContext _executionContext_1 = this.engine.getExecutionContext();
    Resource _resourceModel = _executionContext_1.getResourceModel();
    final Set<Resource> allResources = EMFResource.getRelatedResources(_resourceModel);
    final Consumer<Resource> _function = new Consumer<Resource>() {
      @Override
      public void accept(final Resource r) {
        boolean _notEquals = (!Objects.equal(r, null));
        if (_notEquals) {
          EList<Adapter> _eAdapters = r.eAdapters();
          _eAdapters.add(BatchModelChangeListenerAddon.this.adapter);
        }
      }
    };
    allResources.forEach(_function);
  }
  
  /**
   * When an observer asks for the changes, we process all the notifications gathered for it since the last time.
   */
  public Set<ModelChange> getChanges(final Object addon) {
    final Set<ModelChange> result = new HashSet<ModelChange>();
    final List<Notification> allNotifs = this.changes.get(addon);
    boolean _contains = this.registeredObservers.contains(addon);
    if (_contains) {
      ArrayList<Notification> _arrayList = new ArrayList<Notification>();
      this.changes.put(addon, _arrayList);
    }
    final Map<EObject, Map<EStructuralFeature, List<Notification>>> sortedNotifications = new HashMap<EObject, Map<EStructuralFeature, List<Notification>>>();
    for (final Notification notification : allNotifs) {
      {
        final int eventType = notification.getEventType();
        boolean _and = false;
        boolean _and_1 = false;
        if (!((eventType < Notification.EVENT_TYPE_COUNT) && (notification.getNotifier() instanceof EObject))) {
          _and_1 = false;
        } else {
          boolean _isTouch = notification.isTouch();
          boolean _not = (!_isTouch);
          _and_1 = _not;
        }
        if (!_and_1) {
          _and = false;
        } else {
          Object _feature = notification.getFeature();
          _and = (_feature instanceof EStructuralFeature);
        }
        if (_and) {
          Object _feature_1 = notification.getFeature();
          final EStructuralFeature feature = ((EStructuralFeature) _feature_1);
          Object _notifier = notification.getNotifier();
          final EObject changedObject = ((EObject) _notifier);
          boolean _containsKey = sortedNotifications.containsKey(changedObject);
          boolean _not_1 = (!_containsKey);
          if (_not_1) {
            HashMap<EStructuralFeature, List<Notification>> _hashMap = new HashMap<EStructuralFeature, List<Notification>>();
            sortedNotifications.put(changedObject, _hashMap);
          }
          final Map<EStructuralFeature, List<Notification>> objectsNotifications = sortedNotifications.get(changedObject);
          boolean _containsKey_1 = objectsNotifications.containsKey(feature);
          boolean _not_2 = (!_containsKey_1);
          if (_not_2) {
            ArrayList<Notification> _arrayList_1 = new ArrayList<Notification>();
            objectsNotifications.put(feature, _arrayList_1);
          }
          final List<Notification> fieldsNotifications = objectsNotifications.get(feature);
          fieldsNotifications.add(notification);
        }
      }
    }
    final HashSet<EObject> newObjects = new HashSet<EObject>();
    final HashSet<EObject> removedObjects = new HashSet<EObject>();
    Set<EObject> _keySet = sortedNotifications.keySet();
    for (final EObject object : _keySet) {
      {
        final Map<EStructuralFeature, List<Notification>> featureMap = sortedNotifications.get(object);
        Set<EStructuralFeature> _keySet_1 = featureMap.keySet();
        for (final EStructuralFeature feature : _keySet_1) {
          {
            final List<Notification> notifs = featureMap.get(feature);
            boolean _isMany = feature.isMany();
            boolean _not = (!_isMany);
            if (_not) {
              Notification _head = IterableExtensions.<Notification>head(notifs);
              final Object previousValue = _head.getOldValue();
              Notification _last = IterableExtensions.<Notification>last(notifs);
              final Object newValue = _last.getNewValue();
              if ((feature instanceof EReference)) {
                boolean _notEquals = (!Objects.equal(previousValue, newValue));
                if (_notEquals) {
                  NonCollectionFieldModelChange _nonCollectionFieldModelChange = new NonCollectionFieldModelChange(object, feature);
                  result.add(_nonCollectionFieldModelChange);
                  boolean _isContainment = ((EReference) feature).isContainment();
                  if (_isContainment) {
                    boolean _and = false;
                    boolean _notEquals_1 = (!Objects.equal(previousValue, null));
                    if (!_notEquals_1) {
                      _and = false;
                    } else {
                      _and = (previousValue instanceof EObject);
                    }
                    if (_and) {
                      this.addToRemovedObjects(removedObjects, newObjects, ((EObject) previousValue));
                    }
                    boolean _and_1 = false;
                    boolean _notEquals_2 = (!Objects.equal(newValue, null));
                    if (!_notEquals_2) {
                      _and_1 = false;
                    } else {
                      _and_1 = (newValue instanceof EObject);
                    }
                    if (_and_1) {
                      this.addToNewObjects(removedObjects, newObjects, ((EObject) newValue));
                    }
                  }
                }
              } else {
                boolean _xifexpression = false;
                boolean _equals = Objects.equal(previousValue, null);
                if (_equals) {
                  _xifexpression = (!Objects.equal(newValue, null));
                } else {
                  boolean _equals_1 = previousValue.equals(newValue);
                  _xifexpression = (!_equals_1);
                }
                if (_xifexpression) {
                  NonCollectionFieldModelChange _nonCollectionFieldModelChange_1 = new NonCollectionFieldModelChange(object, feature);
                  result.add(_nonCollectionFieldModelChange_1);
                }
              }
            } else {
              PotentialCollectionFieldModelChange _potentialCollectionFieldModelChange = new PotentialCollectionFieldModelChange(object, feature, notifs);
              result.add(_potentialCollectionFieldModelChange);
              for (final Notification notif : notifs) {
                boolean _and_2 = false;
                if (!(feature instanceof EReference)) {
                  _and_2 = false;
                } else {
                  boolean _isContainment_1 = ((EReference) feature).isContainment();
                  _and_2 = _isContainment_1;
                }
                if (_and_2) {
                  int _eventType = notif.getEventType();
                  switch (_eventType) {
                    case Notification.ADD:
                      Object _newValue = notif.getNewValue();
                      this.addToNewObjects(removedObjects, newObjects, ((EObject) _newValue));
                      break;
                    case Notification.ADD_MANY:
                      Object _newValue_1 = notif.getNewValue();
                      for (final EObject add : ((List<EObject>) _newValue_1)) {
                        this.addToNewObjects(removedObjects, newObjects, add);
                      }
                      break;
                    case Notification.REMOVE:
                      Object _oldValue = notif.getOldValue();
                      this.addToRemovedObjects(removedObjects, newObjects, ((EObject) _oldValue));
                      break;
                    case Notification.REMOVE_MANY:
                      Object _oldValue_1 = notif.getOldValue();
                      for (final EObject remove : ((List<EObject>) _oldValue_1)) {
                        this.addToNewObjects(removedObjects, newObjects, remove);
                      }
                      break;
                  }
                }
              }
            }
          }
        }
      }
    }
    for (final EObject newObject : newObjects) {
      NewObjectModelChange _newObjectModelChange = new NewObjectModelChange(newObject);
      result.add(_newObjectModelChange);
    }
    for (final EObject removedObject : removedObjects) {
      RemovedObjectModelChange _removedObjectModelChange = new RemovedObjectModelChange(removedObject);
      result.add(_removedObjectModelChange);
    }
    return result;
  }
  
  public boolean registerObserver(final Object observer) {
    final boolean res = this.registeredObservers.add(observer);
    if (res) {
      ArrayList<Notification> _arrayList = new ArrayList<Notification>();
      this.changes.put(observer, _arrayList);
    }
    return res;
  }
  
  private void addToNewObjects(final Collection<EObject> removedObjects, final Collection<EObject> newObjects, final EObject object) {
    boolean _notEquals = (!Objects.equal(object, null));
    if (_notEquals) {
      final boolean hasMoved = removedObjects.remove(object);
      if ((!hasMoved)) {
        newObjects.add(object);
      }
    }
  }
  
  private void addToRemovedObjects(final Collection<EObject> removedObjects, final Collection<EObject> newObjects, final EObject object) {
    boolean _notEquals = (!Objects.equal(object, null));
    if (_notEquals) {
      final boolean hasMoved = newObjects.remove(object);
      if ((!hasMoved)) {
        removedObjects.add(object);
      }
    }
  }
  
  @Override
  public void engineAboutToStop(final IBasicExecutionEngine engine) {
    IExecutionContext _executionContext = this.engine.getExecutionContext();
    Resource _resourceModel = _executionContext.getResourceModel();
    final Set<Resource> allResources = EMFResource.getRelatedResources(_resourceModel);
    final Consumer<Resource> _function = new Consumer<Resource>() {
      @Override
      public void accept(final Resource r) {
        EList<Adapter> _eAdapters = r.eAdapters();
        _eAdapters.remove(BatchModelChangeListenerAddon.this.adapter);
      }
    };
    allResources.forEach(_function);
  }
}
